#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <cstdlib>

namespace {

typedef std::vector<std::string> Tokens;

std::vector<std::string> split(const std::string &str,
                               const std::string &delim) {
  std::vector<std::string> result;
  std::string::size_type current_pos = 0;
  std::string::size_type found_pos = 0;
  while ((found_pos = str.find_first_of(delim, current_pos))
         != std::string::npos) {
    if (found_pos > current_pos) {
      result.push_back(str.substr(current_pos, found_pos - current_pos));
    }
    current_pos = found_pos + 1;
  }
  if (str.size() > current_pos) {
    result.push_back(str.substr(current_pos, str.size() - current_pos));
  }
  return result;
}

std::string join(const std::vector<std::string> &tokens,
                 const std::string &delim) {
  std::string result;
  for (size_t i = 0; i < tokens.size(); ++i) {
    if (i != 0) {
      result += delim;
    }
    result += tokens[i];
  }
  return result;
}

void loadExamples(const char *filename,
                  std::vector<Tokens> *output) {
  output->clear();

  std::ifstream ifs(filename);
  if (!ifs) {
    std::cerr << "no such file or directory: " << filename;
    std::exit(-1);
  }

  std::string line;
  while (std::getline(ifs, line)) {
    output->push_back(split(line, "\t "));
  }
}
}  // namespace

int main(int argc, char **argv) {
  if (argc != 3) {
    std::cout << "Usage: " << argv[0] << " <result_data> <golden_data>" << std::endl;
    return -1;
  }

  std::vector<Tokens> results, goldens;
  loadExamples(argv[1], &results);
  loadExamples(argv[2], &goldens);

  if (results.empty() || results.size() != goldens.size()) {
    std::cerr << "file size are differnt or empty" << std::endl;
    return -1;
  }

  size_t correct = 0;
  size_t precision = 0;
  size_t recall = 0;
  size_t all_correct_size = 0;

  for (size_t i = 0; i < results.size(); ++i) {
    const Tokens &golden_tokens = goldens[i];
    const Tokens &result_tokens = results[i];

    const std::string gs = join(golden_tokens, "");
    const std::string rs = join(result_tokens, "");
    if (gs != rs) {
      std::cerr << "Incompatible imput: " << "[" << gs << "] vs [" << rs << "]" << std::endl;
      exit(-1);
    }

    bool all_correct = true;
    size_t golden_index = 0;
    size_t result_index = 0;
    size_t golden_pos = 0;
    size_t result_pos = 0;

    while (golden_index < golden_tokens.size() &&
           result_index < result_tokens.size()) {
      if (golden_pos == result_pos) {
        if (result_tokens[result_index] == golden_tokens[golden_index]) {
          ++correct;
        } else {
          all_correct = false;
        }
        ++precision;
        ++recall;
        result_pos += result_tokens[result_index].size();
        golden_pos += golden_tokens[golden_index].size();
        ++result_index;
        ++golden_index;
      } else if (result_pos < golden_pos) {
        result_pos += result_tokens[result_index].size();
        ++result_index;
        ++precision;
        all_correct = false;
      } else {
        golden_pos += golden_tokens[golden_index].size();
        ++golden_index;
        ++recall;
        all_correct = false;
      }
    }

    while (golden_index < golden_tokens.size()) {
      ++golden_index;
      ++recall;
      all_correct = false;
    }

    while (result_index < result_tokens.size()) {
      ++result_index;
      ++precision;
      all_correct = false;
    }

    if (all_correct) {
      ++all_correct_size;
    }
  }

  const size_t size = results.size();

  const float p = precision == 0 ? 0.0 : 1.0 * correct / precision;
  const float r = recall    == 0 ? 0.0 : 1.0 * correct / recall;
  const float F = (p == 0.0 && r == 0.0) ? 0.0 : 2 * p * r / (p + r);
  const float acc = 1.0 * all_correct_size / results.size();

  std::cout << "Sentence:  " << 100.0 * acc
            << " " << all_correct_size << "/" << results.size() << std::endl;
  std::cout << "Precision: " << 100.0 * p
            << " " << correct << "/" << precision << std::endl;
  std::cout << "Recall:    " << 100.0 * r
            << " " << correct << "/" << recall << std::endl;
  std::cout << "F:         " << 100.0 * F << std::endl;

  return 0;
}
